@@ -1,6 +1,6 @@
 # The R Commander and command logger
 
-# last modified 22 September 06 by J. Fox
+# last modified 6 October 06 by J. Fox
 #   slight changes 12 Aug 04 by Ph. Grosjean 
 
 Commander <- function(){
@@ -11,7 +11,7 @@ Commander <- function(){
       warning("The R Commander is already open.")
       return(invisible(NULL))
     }
-    RcmdrVersion <- "1.2-1"
+    RcmdrVersion <- "1.2-2"
     if (is.SciViews()) return(invisible(svCommander(Version=RcmdrVersion))) # +PhG
     setOption <- function(option, default, global=TRUE) {
         opt <- if (is.null(current[[option]])) default else current[[option]]
@@ -183,8 +183,9 @@ Commander <- function(){
     Menus <- read.table(file.path(etcMenus, "Rcmdr-menus.txt"), as.is=TRUE)
     .Menus <- menus <- list()
     menuItems <- 0
+    oldMenu <- ncol(Menus) == 6
     for (m in 1:nrow(Menus)){
-        install <- Menus[m, 7]
+        install <- if (oldMenu) "" else Menus[m, 7]
         if ((install != "") && (!eval(parse(text=install)))) next  
         if (Menus[m, 1] == "menu") {
             position <- 0
